using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Threading;


namespace ReadyBoostMonitor
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    enum DWMWINDOWATTRIBUTE
    {
        DWMWA_NCRENDERING_ENABLED = 1,      // [get] Is non-client rendering enabled/disabled
        DWMWA_NCRENDERING_POLICY,           // [set] Non-client rendering policy
        DWMWA_TRANSITIONS_FORCEDISABLED,    // [set] Potentially enable/forcibly disable transitions
        DWMWA_ALLOW_NCPAINT,                // [set] Allow contents rendered in the non-client area to be visible on the DWM-drawn frame.
        DWMWA_CAPTION_BUTTON_BOUNDS,        // [get] Bounds of the caption button area in window-relative space.
        DWMWA_NONCLIENT_RTL_LAYOUT,         // [set] Is non-client content RTL mirrored
        DWMWA_FORCE_ICONIC_REPRESENTATION,  // [set] Force this window to display iconic thumbnails.
        DWMWA_FLIP3D_POLICY,                // [set] Designates how Flip3D will treat the window.
        DWMWA_EXTENDED_FRAME_BOUNDS,        // [get] Gets the extended frame bounds rectangle in screen space
        DWMWA_LAST
    }

    enum DWMFLIP3DWINDOWPOLICY
    {
        DWMFLIP3D_DEFAULT,      // Hide or include the window in Flip3D based on window style and visibility.
        DWMFLIP3D_EXCLUDEBELOW, // Display the window under Flip3D and disabled.
        DWMFLIP3D_EXCLUDEABOVE, // Display the window above Flip3D and enabled.
        DWMFLIP3D_LAST
    }

    enum MonitoringMode
    {
        PerformanceCountersNotAvailable,
        Working,
    }

    internal delegate T Func<T>();

    public partial class Window1 : System.Windows.Window
    {
        [DllImport("dwmapi.dll")]
        extern static void DwmSetWindowAttribute(IntPtr hWnd, DWMWINDOWATTRIBUTE dwAttribute, [MarshalAs(UnmanagedType.Bool)] ref bool pvAttribute, uint cbAttribute);

        [DllImport("dwmapi.dll")]
        extern static void DwmSetWindowAttribute(IntPtr hWnd, DWMWINDOWATTRIBUTE dwAttribute, [MarshalAs(UnmanagedType.U4)] ref int pvAttribute, uint cbAttribute);

        static void DwmSetWindowFlip3DPolicy(IntPtr hWnd, DWMFLIP3DWINDOWPOLICY polocy)
        {
            int attr = (int)polocy;
            DwmSetWindowAttribute(hWnd, DWMWINDOWATTRIBUTE.DWMWA_FLIP3D_POLICY, ref attr, (uint) Marshal.SizeOf(typeof(int)) );
        }

        MonitoringMode _mode = MonitoringMode.PerformanceCountersNotAvailable;
        PCs _pc = null;

        readonly ProgressBar[] _progressBars;

        public Window1()
        {
            InitializeComponent();

            try
            {
                _pc = new PCs();
            }
            catch
            {
                this.Close();
                return;
            }

            _progressBars = new [] { progressBar1, progressBar2, progressBar3 };

            this.ToolTip = "GXP[vL[ŏI";

            var timer = new Timer(1000);
            var func = (Func<bool>)this.Update;
            this.Loaded += delegate
                {
                    DwmSetWindowFlip3DPolicy(new WindowInteropHelper(this).Handle, DWMFLIP3DWINDOWPOLICY.DWMFLIP3D_EXCLUDEABOVE);
                    timer.Start();
                };
            timer.Elapsed += delegate
                {
                    // CreateDelegate ʓ|Ȃ̂ŁC_~[ Func delegate `Ă
                    switch (this._mode)
                    {
                        case MonitoringMode.PerformanceCountersNotAvailable:
                            _pc = new PCs();
                            if( _pc.Initialize() )
                            {
                                _mode = MonitoringMode.Working;
                                timer.Interval = 1000;
                            }
                            else
                            {
                                _pc.Dispose();
                                _pc = null;
                                this.maxLabel.Content = "N/A";
                            }
                            break;
                        case MonitoringMode.Working:
                            bool succeeded = (bool)this.Dispatcher.Invoke(DispatcherPriority.Normal, func);
                            if (!succeeded && _pc != null)
                            {
                                _pc.Dispose();
                                _pc = null;
                                timer.Interval = 10000;
                                _mode = MonitoringMode.PerformanceCountersNotAvailable;
                            }
                            break;
                        default:
                            break;
                    }
                };
            this.Unloaded += delegate
                {
                    timer.Dispose();
                };
        }

        class PCs : IDisposable
        {
            public PerformanceCounter RBReadHit;
            public PerformanceCounter RBRead;
            public PerformanceCounter PDRead;
            public PCs()
            {
            }
            public bool Initialize()
            {
                try
                {
                    RBReadHit = new PerformanceCounter("ReadyBoost LbV", "Hit read bytes/sec");
                    RBRead = new PerformanceCounter("ReadyBoost LbV", "Total read bytes/sec");

                    // "C:" ܂ރCX^X쐬
                    PerformanceCounterCategory cat = new PerformanceCounterCategory("PhysicalDisk");
                    foreach (string str in cat.GetInstanceNames())
                    {
                        if (str.Contains("C:"))
                        {
                            PDRead = new PerformanceCounter("PhysicalDisk", "Disk Read Bytes/sec", str);
                        }
                    }
                }
                catch
                {
                    Dispose();
                    return false;
                }
                return true;
            }
            public IEnumerable<double> NextValue()
            {
                yield return RBReadHit.NextValue();
                yield return RBRead.NextValue();
                yield return PDRead.NextValue();
            }

            #region IDisposable Members

            public void Dispose()
            {
                if (RBReadHit != null)
                {
                    RBReadHit.Dispose();
                    RBReadHit = null;
                }
                if (RBRead != null)
                {
                    RBRead.Dispose();
                    RBRead = null;
                }
                if (PDRead != null)
                {
                    PDRead.Dispose();
                    PDRead = null;
                }
            }

            #endregion
        }


        private bool Update()
        {
            try
            {
                List<double> values = new List<double>(_pc.NextValue());
                double max = 0;
                values.ForEach(delegate(double value) { max = Math.Max(max, value); });

                // KɃXP[O
                if (max > 1024 * 1024 * 50) { max = 1024 * 1024 * 100; }
                else if (max > 1024 * 1024 * 10) { max = 1024 * 1024 * 50; }
                else if (max > 1024 * 1024 * 1) { max = 1024 * 1024 * 10; }
                else if (max > 1024 * 256) { max = 1024 * 1024; }
                else { max = 1024 * 256; }

                // xXV
                if (max > (1024 * 1024)) { this.maxLabel.Content = (int)(max / (1024 * 1024)) + " MB/sec"; }
                else if (max > (1024)) { this.maxLabel.Content = (int)(max / 1024) + " KB/sec"; }

                for (int i = 0; i < 3; ++i)
                {
                    _progressBars[i].Maximum = max;
                    _progressBars[i].Value = values[i] > 0 ? values[i] : 0;
                }
            }
            catch
            {
                return false;
            }
            return true;
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            base.OnKeyDown(e);
            if (e.Key == Key.Escape) this.Close();
        }
        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);
            this.DragMove();
        }
    }
}